# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Jesús Corrius i Llavina <jcorrius@fluendo.com>

#from elisa.plugins.poblesec.actions import Action,
from elisa.plugins.poblesec.base.list import GenericListViewMode, \
                                             BaseListController
from elisa.plugins.poblesec.base.preview_list import MenuItemPreviewListController, \
                                    DoubleLineMenuItemPreviewListController
from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController

from elisa.plugins.poblesec.music_library import AlbumsViewMode, \
                                                 ArtistsViewMode, \
                                                 TracksViewMode

from elisa.plugins.database.actions import PlayAlbumAction, \
                                           AlbumMoreOptionsAction, \
                                           PlayTrackAction
from elisa.plugins.database.music_controller import AlbumTracksListController, \
                                                AlbumMoreOptionsController, \
                                                ArtistMoreOptionsController, \
                                                DBTracksViewMode
#from elisa.plugins.database.video_controller import _get_thumbnail

from elisa.plugins.jamendo.models import AlbumModelList, AlbumModel, \
                                         TrackModelList, TrackModel, \
                                         TagModelList, TagModel, \
                                         ArtistModelList, ArtistModel, \
                                         RadioModel, RadioModelList
from elisa.plugins.jamendo.actions import OpenJamendoControllerAction, \
                                          ListJamendoAlbumTracksAction, \
                                          JamendoAlbumMoreOptionsAction, \
                                          ListJamendoArtistAlbumsAction, \
                                          JamendoArtistMoreOptionsAction

from elisa.core import common
from elisa.core.media_uri import MediaUri
from elisa.core.default_config import PICTURES_CACHE
from elisa.core.components.model import Model
from elisa.core.utils.i18n import install_translation
from elisa.core.utils import defer

import os

try:
    from hashlib import md5
except ImportError:
    # hashlib is new in Python 2.5
    from md5 import md5

_ = install_translation('jamendo')
_poblesec = install_translation('poblesec')


class AlbumsMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Albums')
        self.resource = 'elisa.plugins.poblesec.by_album'

class ArtistsMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Artists')
        self.resource = 'elisa.plugins.poblesec.by_artist'

class GenresMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Genres')
        self.resource = 'elisa.plugins.poblesec.by_genre'

class RadiosMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Radios')
        self.resource = 'elisa.plugins.poblesec.radio'

class AlbumsMostPopularMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Most Popular')
        self.resource = 'elisa.plugins.jamendo.most_popular'

class AlbumsPopularThisMonthMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Popular this month')
        self.resource = 'elisa.plugins.jamendo.popular_month'

class AlbumsHigestRatedMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Highest rated')
        self.resource = 'elisa.plugins.jamendo.highest_rated'

class ArtistsMostPopularMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Most Popular')
        self.resource = 'elisa.plugins.jamendo.most_popular'

class ArtistsPopularThisMonthMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Popular this month')
        self.resource = 'elisa.plugins.jamendo.popular_month'

class ArtistsHigestRatedMenuEntryModel(Model):
    def __init__(self):
        super(Model, self).__init__()
        self.title = _('Highest rated')
        self.resource= 'elisa.plugins.jamendo.highest_rated'


def jamendo_decorator(controller):
    controller.append_plugin( 'jamendo',
                              _('Jamendo'),
                              '/poblesec/jamendo' )
    return defer.succeed(None)


class JamendoController(BaseListController):
    def initialize(self, jamendo_model = None):
        self.jamendo_model = jamendo_model
        self.current_album = None
        return super(JamendoController, self).initialize()

    def populate_model(self):
        model = []

        def add_menu_entries():
            menu_entries_models = [AlbumsMenuEntryModel(),
                                   ArtistsMenuEntryModel(),
                                   GenresMenuEntryModel(),
                                   RadiosMenuEntryModel()]
            model.extend(menu_entries_models)
            return defer.succeed(model)

        def add_albums_menu_entries():
            menu_entries_models = [AlbumsMostPopularMenuEntryModel(),
                                   AlbumsPopularThisMonthMenuEntryModel(),
                                   AlbumsHigestRatedMenuEntryModel()]
            model.extend(menu_entries_models)
            return defer.succeed(model)
            
        def add_artists_menu_entries():
            menu_entries_models = [ArtistsMostPopularMenuEntryModel(),
                                   ArtistsPopularThisMonthMenuEntryModel(),
                                   ArtistsHigestRatedMenuEntryModel()]
            model.extend(menu_entries_models)
            return defer.succeed(model)

        def insert_items(result, items):
            if isinstance(self.jamendo_model, AlbumModel):
                self.current_album = items
            model.extend(items.list)
            return defer.succeed(model)

        if not self.jamendo_model:
            return add_menu_entries()
        elif isinstance(self.jamendo_model, AlbumsMenuEntryModel):
            return add_albums_menu_entries()
        elif isinstance(self.jamendo_model, ArtistsMenuEntryModel):
            return add_artists_menu_entries()
        elif isinstance(self.jamendo_model, GenresMenuEntryModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+url/tag/json/?n=50&order=rating_desc")
        elif isinstance(self.jamendo_model, RadiosMenuEntryModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+image+url/radio/json/?n=50&nshuffle=200&order=rating_desc&imagesize=600")

        items, deferred = common.application.resource_manager.get(uri)
        if isinstance(self.jamendo_model, RadiosMenuEntryModel):
            def order_items(result, items):
                items.list = sorted(items.list, key=lambda obj: obj.title.lower())
                return result, items
            deferred.addCallback(order_items, items)
        deferred.addCallback(insert_items, items)

        return deferred

    def create_actions(self):
        if isinstance(self.jamendo_model, AlbumsMenuEntryModel)or \
                isinstance(self.jamendo_model, GenresMenuEntryModel):
            path = '/poblesec/jamendo/albums'
        elif isinstance(self.jamendo_model, ArtistsMenuEntryModel): 
            path = '/poblesec/jamendo/artists'
        elif isinstance(self.jamendo_model, RadiosMenuEntryModel):
            path = '/poblesec/jamendo/tracks'
        else:
            path = '/poblesec/jamendo'

        return OpenJamendoControllerAction(self, path), []


class JamendoViewMode(GenericListViewMode):
    """
    Implementation of the common view modes API.
    """

    def get_label(self, item):
        if item.title:
            return defer.succeed(item.title)
        else:
            return defer.succeed(_('[Unnamed]'))

    def get_default_image(self, item):
        return 'elisa.plugins.poblesec.glyphs.small.music'

    def get_image(self, item, theme):
        return None


class JamendoPreviewListController(JamendoController, MenuItemPreviewListController):
    view_mode = JamendoViewMode
    item_widget_kwargs = {'with_artwork_box': False}
    crumbs_logo = "elisa.plugins.jamendo.logo"


class JamendoAlbumsController(BaseListController):

    empty_label = _('There are no albums in this section')

    def initialize(self, jamendo_model = None):
        self.jamendo_model = jamendo_model
        return super(JamendoAlbumsController, self).initialize()

    def populate_model(self):
        model = []

        def insert_items(result, items):
            if isinstance(self.jamendo_model, AlbumModel):
                self.current_album = items
            model.extend(items.list)
            return defer.succeed(model)

        if isinstance(self.jamendo_model, AlbumsMostPopularMenuEntryModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+url+image+artist_name/album/json/?n=50&nshuffle=200&order=rating_desc&imagesize=600")
        elif isinstance(self.jamendo_model, AlbumsPopularThisMonthMenuEntryModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+url+image+artist_name/album/json/?n=50&nshuffle=200&order=ratingmonth_desc&imagesize=600")
        elif isinstance(self.jamendo_model, AlbumsHigestRatedMenuEntryModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+url+image+artist_name/album/json/?n=50&nshuffle=200&order=starred_desc&imagesize=600")
        elif isinstance(self.jamendo_model, ArtistModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+url+image+artist_name/album/json/?artist_id=%s&n=50&imagesize=600" % self.jamendo_model.jamendo_id)
        elif isinstance(self.jamendo_model, TagModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+url+image+artist_name/album/json/?tag_idstr=%s&n=50&nshuffle=200&order=starred_desc&imagesize=600" % self.jamendo_model.title)

        items, deferred = common.application.resource_manager.get(uri)
        deferred.addCallback(insert_items, items)

        return deferred

    def create_actions(self):
        # Default action: play all tracks in the album.
        default = PlayAlbumAction(self)
        # Contextual actions: list tracks, more options.
        list_tracks = ListJamendoAlbumTracksAction(self)
        more = JamendoAlbumMoreOptionsAction(self)
        return default, [list_tracks, more]


class JamendoAlbumsPreviewListController(JamendoAlbumsController, DoubleLineMenuItemPreviewListController):
    view_mode = AlbumsViewMode
    crumbs_logo = 'elisa.plugins.jamendo.logo'


class JamendoAlbumTracksListController(AlbumTracksListController):
    def node_renderer(self, item, widget):
        trash_letters = " .,-_"
        title = item.title.strip(trash_letters)
        widget.item_widget.text.label = title


class JamendoAlbumMoreOptionsController(AlbumMoreOptionsController):
    pass


class JamendoArtistsController(BaseListController):

    def initialize(self, jamendo_model = None):
        self.jamendo_model = jamendo_model
        return super(JamendoArtistsController, self).initialize()

    def populate_model(self):
        model = []

        def insert_items(result, items):
            model.extend(items.list)
            return defer.succeed(model)

        if isinstance(self.jamendo_model, ArtistsMostPopularMenuEntryModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+image/artist/json/?n=50&nshuffle=200&order=rating_desc&imagesize=600")
        elif isinstance(self.jamendo_model, ArtistsPopularThisMonthMenuEntryModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+image/artist/json/?n=50&nshuffle=200&order=ratingmonth_desc&imagesize=600")
        elif isinstance(self.jamendo_model, ArtistsHigestRatedMenuEntryModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+image/artist/json/?n=50&nshuffle=200&order=starred_desc&imagesize=600")

        items, deferred = common.application.resource_manager.get(uri)
        deferred.addCallback(insert_items, items)

        return deferred

    def create_actions(self):
        # Default action: play all tracks in the album.
        default = ListJamendoArtistAlbumsAction(self)
        # Contextual actions: list tracks, more options.
        more = JamendoArtistMoreOptionsAction(self)
        return default, [more]


class JamendoArtistsViewMode(ArtistsViewMode):
    def get_image(self, item, theme):
        if item.image_uri is not None and len(item.image_uri) < 1:
            item.image_uri = None
        return super(JamendoArtistsViewMode, self).get_image(item, theme)


class JamendoArtistsPreviewListController(JamendoArtistsController, MenuItemPreviewListController):
    view_mode = JamendoArtistsViewMode
    crumbs_logo = 'elisa.plugins.jamendo.logo'


class JamendoArtistMoreOptionsController(ArtistMoreOptionsController):
    def populate_info(self):
        if self.artist.image_uri is not None and len(self.artist.image_uri) < 1:
            self.artist.image_uri = None
        return super(JamendoArtistMoreOptionsController, self).populate_info()


class JamendoTracksController(BaseListController):
    def initialize(self, jamendo_model = None):
        self.jamendo_model = jamendo_model
        return super(JamendoTracksController, self).initialize()

    def populate_model(self):
        model = []

        def insert_items(result, items):
            model.extend(items.list)
            return defer.succeed(model)

        if isinstance(self.jamendo_model, RadioModel):
            uri = MediaUri("http://api.jamendo.com/get2/id+name+url+stream+artist_name/track/json/radio_track_inradioplaylist+track_album+album_artist/?order=numradio_asc&radio_id=%s" % self.jamendo_model.jamendo_id)

        items, deferred = common.application.resource_manager.get(uri)
        deferred.addCallback(insert_items, items)

        return deferred

    def create_actions(self):
        default = PlayTrackAction(self)
        return default, []


class JamendoTracksPreviewListController(JamendoTracksController, 
                                    DoubleLineMenuItemPreviewListController):
    view_mode = DBTracksViewMode
    crumbs_logo = 'elisa.plugins.jamendo.logo'


def use_me_hook(frontend):
    """
    'Use me' hook that takes the user to the Internet Media/Music/Jamendo
    section.

    @param frontend: the current frontend (poblesec)
    @type frontend:  L{elisa.plugins.pigment.pigment_frontend.PigmentFrontend}

    @return:         a deferred fired when the action is complete
    @rtype:          L{twisted.internet.defer.Deferred}
    """
    browser = frontend.retrieve_controllers('/poblesec/browser')[0]

    paths = [('/poblesec/internet_menu', _poblesec('INTERNET MEDIA'), {}),
             ('/poblesec/music/internet', _poblesec('Music'), {}),
             ('/poblesec/jamendo', _('Jamendo'), {})]

    return browser.navigate(paths)