# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Jesús Corrius i Llavina <jcorrius@fluendo.com>

from elisa.core.components.resource_provider import ResourceProvider
from elisa.plugins.http_client.http_client import ElisaHttpClient
from elisa.plugins.http_client.extern.client_http import ClientRequest

from elisa.plugins.jamendo.models import AlbumModelList, AlbumModel, \
                                         TrackModelList, TrackModel, \
                                         TagModelList, TagModel, \
                                         ArtistModelList, ArtistModel, \
                                         RadioModel, RadioModelList

from elisa.plugins.base.models.image import ImageModel
from elisa.plugins.base.models.media import RawDataModel

from elisa.core.utils import defer
from twisted.internet import task
from twisted.web2 import responsecode
from twisted.web2.stream import BufferedStream, MemoryStream

from simplejson.decoder import JSONDecoder

SERVER_NAME = 'api.jamendo.com'
SERVER_PORT = 80

class ErrorInResponse(Exception):
    pass

"""
Map uri-path unit to the corresponding model
"""

UNIT_TO_MODEL = {'album': AlbumModelList, 'track': TrackModelList, 'tag': TagModelList, 'artist': ArtistModelList, 'radio': RadioModelList }

ALWAYSMAP = {'id' : 'jamendo_id' }

PARSING_MAP = {
    AlbumModelList: ({'': AlbumModel}, {}),
    AlbumModel: ({}, {'name' : 'name', 'url' : 'uri', 'image' : 'cover_uri', 'artist_name' : 'artist', }),
    TrackModelList: ({'': TrackModel}, {}),
    TrackModel: ({}, {'name' : 'title', 'url' : 'uri', 'stream' : 'playable_uri', 'artist_name' : 'artist', 'album_name' : 'album',}),
    TagModelList: ({'': TagModel}, {}),
    TagModel: ({}, {'name' : 'title', 'url' : 'uri', }),
    ArtistModelList: ({'': ArtistModel}, {}),
    ArtistModel: ({}, {'name' : 'name', 'image' : 'image_uri', }),
    RadioModelList: ({'': RadioModel}, {}),
    RadioModel: ({}, {'name' : 'title', 'url' : 'uri' }),
              }

class JamendoResourceProvider(ResourceProvider):
    """
    This resource provider makes the content of jamendo available inside elisa.
    """
    
    supported_uri = 'http://api.jamendo.com/get2/*|http://imgjam.com/*'
    
    def __init__(self):
        super(JamendoResourceProvider, self).__init__()
        self._http_client = ElisaHttpClient(SERVER_NAME, SERVER_PORT)
        self._data_connections = {}
        self.decoder = JSONDecoder()
        
    def clean(self):
        """
        """
        def close_connections(result):
            dfrs = [self._http_client.close()]
            for client in self._data_connections.values():
                dfrs.append(client.close())
            return defer.DeferredList(dfrs, consumeErrors=True)

        dfr = super(JamendoResourceProvider, self).clean()
        dfr.addCallback(close_connections)
        return dfr
        
    def get(self, uri, context_model):
        """
        """
        if uri.host != SERVER_NAME:
            return self._get_raw(uri)

        model, rip_out = self.get_model_for_uri_path(uri.path)
        if not model:
            return None, defer.fail(NotImplementedError(uri))
        
        dfr = self._send_simple_request(uri)
        dfr.addCallback(self._parse_response)
        dfr.addCallback(self.decoder.decode)

        dfr.addCallback(self.parse_data_into_model, model)
        return model, dfr
    
    def get_model_for_uri_path(self, uri_path):
        """
        Create the model corresponding to the given URI path.
        """
        splitted_path = uri_path.split('/')
        splitted_path.pop(0) # this is always empty
        
        # make sure this is for for the version 2 of the API:
        assert splitted_path.pop(0) == 'get2'
        splitted_path.pop(0) # this is the list of fields
        
        unit = splitted_path.pop(0)
        
        try:
            return UNIT_TO_MODEL[unit](), unit
        except KeyError:
            return None, None
    
    def _send_simple_request(self, uri):
        req = ClientRequest('GET', uri, None, None)
        return self._http_client.request_full(req)
    
    def _parse_response(self, response):
        if response.code != responsecode.OK:
            raise ErrorInResponse(response.code)

        buf_stream = BufferedStream(response.stream)

        read_dfr = defer.maybeDeferred(buf_stream.readExactly)
        return read_dfr
    
    def parse_data_into_model(self, data, model):
        """
        """
        def setter(model, key, mapped_key, value):
            setattr(model, mapped_key, value)

        def to_obj_parser(model, key, values, klass):
            if not values:
                # the values we should parse are actually an empty list or even
                # worse a NoneType. We should not create any model(s) in that
                # case
                return

            if isinstance(values, list):
                # we have a list of them, so parse each one of them
                dfr = task.coiterate(self._parse_list_of_data(values, \
                       getattr(model, key), klass))
            else:
                obj = klass()
                setattr(model, key, obj)
                dfr = self.parse_data_into_model(values, obj)
            return dfr
        
        def parse_attrs(result, attribute_map):
            dfr = task.coiterate(self._parse_simple(model, \
                    data, attribute_map, setter))
            return dfr

        def ret_model(result, model):
            return model
        
        list_map, attribute_map = PARSING_MAP[type(model)]
        try:
            klass = list_map['']
            dfr = task.coiterate(self._parse_list_of_data(data, getattr(model, 'list'), klass))
        except KeyError:
            dfr = task.coiterate(self._parse_simple(model, list_map, data, to_obj_parser))
            dfr.addCallback(parse_attrs, attribute_map)
            dfr.addCallback(ret_model, model)

        return dfr
                   
    def _parse_list_of_data(self, ref_list, list_to_fill, klass):
        for data in ref_list:
            obj = klass()
            list_to_fill.append(obj)
            yield self.parse_data_into_model(data, obj)
            
    def _parse_simple(self, model, data, key_map, callback):
        for key, value in data.iteritems():
            try:
                mapped = key_map[key]
            except KeyError:
                try:
                    mapped = ALWAYSMAP[key]
                except KeyError:
                    yield None
                    continue

            yield callback(model, key, mapped, value)
            
    def _get_raw(self, uri):
        host = uri.host
        try:
            client = self._data_connections[host]
        except KeyError:
            # no client for that host yet, create one
            client = ElisaHttpClient(host, uri.port or 80)
            self._data_connections[host] = client

        model = RawDataModel()

        dfr = client.request(uri)
        dfr.addCallback(self._parse_response)
        dfr.addCallback(self._parse_raw, model)
        return model, dfr
    
    def _parse_raw(self, response_data, model):
        model.data = response_data
        model.size = len(response_data)
        return model
        
    